<?php

App::uses('AppBehavior', 'Model/Behavior');

class CustomFieldsBehavior extends AppBehavior
{
    /**
     * define an afterFind to retrieve hte custom field data for the item
     *
     * @param Model $model Model using this behavior
     * @param mixed $results The results of the find operation
     * @param bool $primary Whether this model is being queried directly (vs. being queried as an association)
     * @return mixed An array value will replace the value of $results - any other value will be ignored.
     */
    public function afterFind(Model $Model, $results, $primary = false)
    {
        // get all the model ids
        $modelIds = Hash::extract($results, '{n}.' . $Model->alias . '.' . $Model->primaryKey);

        if (empty($modelIds)) {
            return $results;
        }

        // get all the fields and data where id IN model_id
        $Field = EvClassRegistry::init('EvCustomFields.Field');
        $fields = $Field->find(
            'all',
            array(
                'conditions' => array(
                    'model' => EvClassRegistry::getNameFromModel($Model),
                    'model_id' => $modelIds
                ),
                'order' => 'Field.sequence ASC'
            )
        );

        if (! empty($fields)) {

            $customFields = Hash::combine(
                $fields,
                '{n}.Field.id',
                '{n}',
                '{n}.Field.model_id'
            );

            // loop the results and assign the field data to the result
            foreach ($results as $key => $item) {

                if (empty($item[$Model->alias][$Model->primaryKey])) {
                    continue;
                }

                if (! empty($customFields[$item[$Model->alias][$Model->primaryKey]])) {

                    $results[$key]['CustomFields'] = array_map(
                        function($customField) use ($Field) {

                            if (! empty($customField[$Field->alias]['options'])) {

                                $customField[$Field->alias]['options'] = json_decode($customField[$Field->alias]['options'], true);
                            }
                            if (! empty($customField[$Field->alias]['validation'])) {

                                $customField[$Field->alias]['validation'] = json_decode($customField[$Field->alias]['validation'], true);
                            }

                            return $customField;
                        },
                        $customFields[$item[$Model->alias][$Model->primaryKey]]
                    );
                }
            }
        }

        return $results;
    }


    /**
     * process any options and validation into json for storage
     *
     * @param Model $model Model using this behavior
     * @param array $options Options passed from Model::save().
     * @return mixed False if the operation should abort. Any other result will continue.
     * @see Model::save()
     */
    public function beforeSave(Model $Model, $options = array())
    {
        $return = parent::beforeSave($Model, $options);

        if (! empty($Model->data[$Model->alias]['CustomFields'])) {

            foreach ($Model->data[$Model->alias]['CustomFields'] as $key => $field) {

                if (! empty($field['Field']['options'])) {
                    $Model->data[$Model->alias]['CustomFields'][$key]['Field']['options'] = json_encode($field['Field']['options']);
                } else {
                    $Model->data[$Model->alias]['CustomFields'][$key]['Field']['options'] = '';
                }

                if (! empty($field['Field']['validation'])) {
                    $Model->data[$Model->alias]['CustomFields'][$key]['Field']['validation'] = json_encode($field['Field']['validation']);
                } else {
                    $Model->data[$Model->alias]['CustomFields'][$key]['Field']['validation'] = '';
                }
            }
        }

        return $return;
    }


    /**
     * Save the custom fields
     *
     * @param Model $model Model using this behavior
     * @param bool $created True if this save created a new record
     * @param array $options Options passed from Model::save().
     * @return bool
     * @see Model::save()
     */
    public function afterSave(Model $Model, $created, $options = array())
    {
        $return = parent::afterSave($Model, $created, $options);
        $result = true;

        $Field = EvClassRegistry::init('EvCustomFields.Field');

        // process any custom field deletions
        if (! empty($Model->data[$Model->alias]['CustomField']['toDelete'])) {

            $ids = explode(',', $Model->data[$Model->alias]['CustomField']['toDelete']);
            $ids = ArrayUtil::clearArray($ids);

            $Field->deleteAll(
                array(
                    'Field.id' => $ids
                ),
                true,
                true
            );
        }

        // add / update any custom fields
        if (! empty($Model->data[$Model->alias]['CustomFields'])) {

            // setup the model / model id
            $modelInfo = array(
                'model' => EvClassRegistry::getNameFromModel($Model),
                'model_id' => $Model->data[$Model->alias][$Model->primaryKey]
            );

            foreach ($Model->data[$Model->alias]['CustomFields'] as $key => $field) {

                $Model->data[$Model->alias]['CustomFields'][$key]['Field']['model'] = $modelInfo['model'];
                $Model->data[$Model->alias]['CustomFields'][$key]['Field']['model_id'] = $modelInfo['model_id'];
            }

            $result = $Field->saveMany(
                $Model->data[$Model->alias]['CustomFields']
            );
        }

        return ($return && $result);
    }
}
